public class ThreadTest {
    public static void printArray( int[] array ) {
	System.out.print( "[ " );
	for( int x = 0; x < array.length - 1; x++ ) {
	    System.out.print( array[ x ] + ", " );
	}
	System.out.println( array[ array.length - 1 ] + " ]" );
    }

    public static int[] makeArray( int size ) {
	int[] retval = new int[ size ];
	for( int x = 0; x < size; x++ ) {
	    retval[ x ] = x;
	}
	return retval;
    }

    public static AddTwoRunnable[] makeArgs( int[] array,
					     int numThreads ) {
	AddTwoRunnable[] retval = new AddTwoRunnable[ numThreads ];
	int numPerThread = array.length / numThreads;
	int start = 0;
	int end = numPerThread - 1;

	for( int x = 0; x < numThreads - 1; x++ ) {
	    retval[ x ] = new AddTwoRunnable( array,
					      start,
					      end );
	    start = end + 1;
	    end = start + numPerThread - 1;
	}
	retval[ numThreads - 1 ] = new AddTwoRunnable( array,
						       start,
						       array.length - 1 );
	
	return retval;
    }
	
    public static void runThreads( int[] array,
				   int numThreads ) {
	AddTwoRunnable[] args = makeArgs( array, numThreads );
	Thread[] threads = new Thread[ numThreads ];
	for( int x = 0; x < numThreads; x++ ) {
	    Thread thread = new Thread( args[ x ] );
	    thread.start();
	    threads[ x ] = thread;
	}
	
	// at this point the threads re running in parallel
	for( int x = 0; x < numThreads; x++ ) {
	    try {
		threads[ x ].join();
	    } catch ( InterruptedException e ) {}
	}
    }

    public static void usage() {
	System.out.println( "Takes the following params:\n" +
			    "-The length of the array to make\n" +
			    "-The number of threads to execute" );
    }

    public static void main( String[] args ) {
	if ( args.length != 2 ) {
	    usage();
	    System.exit( 1 );
	}

	try {
	    boolean error = false;
	    int size = Integer.parseInt( args[ 0 ] );
	    int numThreads = Integer.parseInt( args[ 1 ] );
	    
	    if ( size <= 0 ) {
		System.err.println( "The size must be positive." );
		error = true;
	    } 
	    if ( numThreads <= 0 ) {
		System.err.println( "The number of threads to create must be" +
				    " positive." );
		error = true;
	    }
	    if ( numThreads > size ) {
		System.err.println( "The number of threads must be <= the " +
				    "size." );
		error = true;
	    }

	    if ( !error ) {
		int[] array = makeArray( size );
		runThreads( array,
			    numThreads );
		printArray( array );
	    }
	} catch ( NumberFormatException e ) {
	    System.err.println( e );
	}
    }
}

	   